@@ -8,12 +8,11 @@ Parse::Lotus123::WK4 - extract data from Lotus 1-2-3 .wk4 files
 
 This module extracts data from Lotus 1-2-3 .wk4 files.
 
-=head1 NO DOCUMENTATION
+=head1 SYNOPSIS
 
 Procedural API:
 Parse::Lotus123::WK4::parse takes a filehandle and returns a three-dimensional arrayref.
-
-See the source code to wk42csv for a working example.
+See the source code to wk42csv for more information.
 
 =head1 SOURCES
 
@@ -21,11 +20,11 @@ Description of WK4 format:
 L<http://www.mettalogic.uklinux.net/tim/l123/l123r4.html>
 
 Method for decoding IEEE 80-bit floats:
-L<http://www.perlmonks.org/?node=586923>
+L<http://www.perlmonks.org/?node_id=587419>
 
 =head1 BUGS
 
-This code is experimental, not documented and not properly tested.
+This code is experimental, not properly tested and not suitable for production use.
 
 =head1 NO WARRANTY
 
@@ -33,12 +32,14 @@ This code comes with ABSOLUTELY NO WARRANTY of any kind.
 
 =head1 AUTHOR
 
-Copyright 2008 Reckon LLP and Franck Latrémolière.
+Written by Franck Latremoliere.
+Copyright (c) 2008 Reckon LLP.
 L<http://www.reckon.co.uk/staff/franck/>
 
 =head1 LICENCE
 
-This is free software; you can redistribute it and/or modify it under the same terms as Perl.
+This program is free software; you can use, redistribute and/or modify it under the same terms as Perl itself
+(Artistic Licence or GNU GPL).
 
 =cut
 
@@ -48,7 +49,7 @@ use strict;
 
 BEGIN {
 
-    $Parse::Lotus123::WK4::VERSION = '0.09';
+    $Parse::Lotus123::WK4::VERSION = '0.081';
 
 # test for float endianness using little-endian 33 33 3b f3, which is a float code for 1.4
     
@@ -64,34 +65,31 @@ BEGIN {
 }
 
 sub decode_lotus_weirdness {
-    my $h = unpack 's', pack 'S', $_[0];
-    return $h / 2 unless $h & 1;
+    my $h = $_[0];
+    return $h / 2 if !( $h & 1 );
+    my $ld = $h >> 4;
     my $sw = $h & 0x0f;
-    {
-        use integer; # this makes the right-shift operator signed for the block
-        $h >>= 4;
-    }
-    return $h * 5000  if $sw == 0x1;
-    return $h * 500   if $sw == 0x3;
-    return $h / 20    if $sw == 0x5;
-    return $h / 200   if $sw == 0x7;
-    return $h / 2000  if $sw == 0x9;
-    return $h / 20000 if $sw == 0xb;
-    return $h / 16    if $sw == 0xd;
-    return $h / 64    if $sw == 0xf;
+    return $ld * 5000  if $sw == 0x1;
+    return $ld * 500   if $sw == 0x3;
+    return $ld / 20    if $sw == 0x5;
+    return $ld / 200   if $sw == 0x7;
+    return $ld / 2000  if $sw == 0x9;
+    return $ld / 20000 if $sw == 0xb;
+    return $ld / 16    if $sw == 0xd;
+    return $ld / 64    if $sw == 0xf;
 }
 
 sub decode_float80 {
-    my( $discard, $mantissa, $hidden, $exponent, $sign ) =
-        unpack 'a11 a52 a1 a15 a1', $_[ 0 ];
+    my ( $discard, $mantissa, $hidden, $exponent, $sign ) =
+      unpack 'a11 a52 a1 a15 a1', $_[0];
     $exponent = unpack( 'v', pack 'b15', $exponent ) - 16383 + 1023;
-    ($exponent, $mantissa) = (32767, '0' x 52)
-        if $exponent < 0 || $exponent > 2047;
+    $exponent = 32767, $mantissa = '0' x 52
+      if $exponent < 0 || $exponent > 2047;
     $exponent = unpack 'b11', pack 'v', $exponent;
-    my $bits64 = pack 'b64', $mantissa . $exponent . $sign;
-    $bits64 = pack 'a' x 8, reverse unpack 'a' x 8, pack 'b64', $bits64
+     my $bits64 = $mantissa . $exponent . $sign;
+    $bits64 = pack 'a'x8, reverse unpack 'a' x 8, pack 'b64', $bits64
       if $Parse::Lotus::WK4::bigEndian;
-    unpack 'd', $bits64;
+    return unpack 'd', $bits64;
 }
 
 sub parse($) {
@@ -102,7 +100,8 @@ sub parse($) {
         my $read = read ($fh, my $byt, $len);
         if ( $read != $len ) {
             # warn "Could not read $len bytes";
-            # no need to warn the user: we are probably just at the end of the file
+            # no need to warn the user:
+            # we are probably just at the end of the file and got the structure wrong
         }
         elsif ( $code == 0x16 ) {
             my ( $row, $sheet, $col, $align, $text ) = unpack( 'vCCCA*', $byt );
